//////////////////////////////////////////////////////////////////////////////////////////////
// Clase para resolver un sistema lineal tridiagonal                                        //
//                                                                                          //
// Anlisis y Diseo y Programacin:                                                        //
//                                                                                          //
// Nombre:   Antonio Carrillo Ledesma                                                       //
// E-mail:   acl@www.mmc.igeofcu.unam.mx                                                    //
// Pgina:   http://www.mmc.igeofcu.unam.mx/acl                                             //
//                                                                                          //
//                                                                                          //
// Este programa es software libre. Puede redistribuirlo y/o modificarlo                    //
// bajo los trminos de la Licencia Pblica General de GNU segn es                         //
// publicada por la Free Software Foundation, bien de la versin 2 de                       //
// dicha Licencia o bien (segn su eleccin) de cualquier versin                           //
// posterior.                                                                               //
//                                                                                          //
// Este programa se distribuye con la esperanza de que sea til, pero SIN                   //
// NINGUNA GARANTA, incluso sin la garanta MERCANTIL implcita o sin                      //
// garantizar la CONVENIENCIA PARA UN PROPSITO PARTICULAR. Vase la                        //
// Licencia Pblica General de GNU para ms detalles.                                       //
//                                                                                          //
// Debera haber recibido una copia de la Licencia Pblica General junto                    //
// con este programa. Si no ha sido as, escriba a la Free Software                         //
// Foundation, Inc., en 675 Mass Ave, Cambridge, MA 02139, EEUU.                            //
//                                                                                          //
//                                                                                          //
//////////////////////////////////////////////////////////////////////////////////////////////


#include "ResuelveTridiagonal.hpp"


// Resuelve el sistema lineal
void ResuelveTridiagonal::resuelve(void)
{
#ifdef DEPURAR
   if (!M->matrizCuadrada()) error("matriz no cuadrada");
#endif

   int i;
   ldouble XX, XY;

   // Factorizacin LU
   for (i = 0; i < M->renglones()-1; i++)
   {
#ifdef DEPURAR
      if (M->retorna(i,i) == 0.0) error("Diagonal con valores cero");
#endif
      XX = M->retorna(i,i+1) / M->retorna(i,i);
      M->asigna(i,i+1,XX);
      XY = M->retorna(i+1,i+1) - M->retorna(i+1,i) * XX;
      M->asigna(i+1,i+1,XY);
   }

   // Resuelve Lx=y
#ifdef DEPURAR
   if (M->retorna(0,0) == 0.0) error("Diagonal con valores cero");
#endif
   X->asigna(0,B->retorna(0) / M->retorna(0,0));
   for (i = 1; i < M->renglones(); i++)
   {
#ifdef DEPURAR
      if (M->retorna(i,i) == 0.0) error("Diagonal con valores cero");
#endif
      X->asigna(i,(B->retorna(i) - M->retorna(i,i-1) * X->retorna(i - 1)) / M->retorna(i,i));
   }
   // Resuleve Ux=y
   for (i = M->renglones() - 2; i >= 0; i--) X->asigna(i,X->retorna(i) - X->retorna(i+1) * M->retorna(i,i+1));
}


